# -*- coding: utf-8 -*-
#
# FirstAlexaSkills documentation build configuration file, created by
# sphinx-quickstart on Tue Mar 14 21:58:22 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, u'/Users/vlasto/src/FirstAlexaSkills/src/FirstAlexaSkills')
import sys
import pkg_resources
try:
    import sphinx_rtd_theme
except ImportError:
    sphinx_rtd_theme = None

try:
    from sphinxcontrib import spelling
except ImportError:
    spelling = None

# -- General configuration ------------------------------------------------
# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.todo', 'sphinx.ext.viewcode']

if spelling is not None:
    extensions.append('sphinxcontrib.spelling')

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'FirstAlexaSkills'
copyright = u'2017, Vlastimil Pis'
author = u'Vlastimil Pis'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
try:
    release = pkg_resources.get_distribution('FirstAlexaSkills').version
except pkg_resources.DistributionNotFound:
    print 'To build the documentation, The distribution information of FirstAlexaSkills'
    print 'Has to be available.  Either install the package into your'
    print 'development environment or run "setup.py develop" to setup the'
    print 'metadata.  A virtualenv is recommended!'
    sys.exit(1)
del pkg_resources

version = '.'.join(release.split('.')[:2])
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#

if sphinx_rtd_theme:
    html_theme = "sphinx_rtd_theme"
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
else:
    html_theme = "default"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'FirstAlexaSkillsdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'FirstAlexaSkills.tex', u'FirstAlexaSkills Documentation', u'Author', 'manual')]

# -- Options for manual page output ---------------------------------------
# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'firstalexaskills', u'FirstAlexaSkills Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------
# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'FirstAlexaSkills', u'FirstAlexaSkills Documentation',
     author, 'FirstAlexaSkills', 'One line description of project.',
     'Miscellaneous'),
]


# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = author
epub_copyright = copyright

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']



#!/usr/bin/env python

import os
import sys
import logging
import shutil
import json
import copy
import collections
import tempfile
import time
import boto3
import pkg_resources

logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - '
                           '%(message)s', datefmt='%Y/%m/%d %I:%M:%S %p',
                    level=logging.ERROR)

cur_dir = os.path.dirname(os.path.realpath(__file__))


def get_skills_archive_fn():
    resource_package = "FirstAlexaSkills"
    # Do not use os.path.join()!
    resource_path = '/'.join(('data', 'example_skills.zip'))
    my_filename = pkg_resources.resource_filename(resource_package,
                                                  resource_path)
    return my_filename


def get_eventtemplate_fn():
    resource_package = "FirstAlexaSkills"
    # Do not use os.path.join()!
    resource_path = '/'.join(('data', 'event_template.json'))
    my_filename = pkg_resources.resource_filename(resource_package,
                                                  resource_path)
    return my_filename


def get_account_id(profile_name, region=None):
    owner_id = None
    session = boto3.Session(profile_name=profile_name, region_name=region)
    client = session.client('ec2')
    response = client.describe_security_groups()
    if 'SecurityGroups' in response:
        owner_id = response['SecurityGroups'][0]['OwnerId']
    return owner_id


def get_execrole_arn(execrole_name, profile_name, region=None):
    account_id = get_account_id(profile_name, region)
    execrole_arn = 'arn:aws:iam::' + account_id + ':role/' + execrole_name
    return execrole_arn


def list_lambda_functions(client):
    function_name_list = []
    # Create a reusable Paginator
    paginator = client.get_paginator('list_functions')
    # Create a PageIterator from the Paginator
    page_iterator = paginator.paginate()
    for page in page_iterator:
        for function in page['Functions']:
            function_name_list.append(function['FunctionName'])
    return function_name_list


def create_zipfile(dir_path):
    zip_dir = tempfile.mkdtemp()
    ext = 'zip'
    function_zip = "function"  # will produce a zip archive
    shutil.make_archive(os.path.join(zip_dir, function_zip), ext, dir_path)
    return os.path.join(zip_dir, function_zip + '.' + ext)


def create_lambda(client, function_name, dir_path, role):
    # Todo: infer environment from file suffix
    # Todo: add support for packing virtualenv for Python
    logger = logging.getLogger(__name__)
    zip_file = ""
    fun_arn = None
    try:
        runtime = 'python2.7'
        zip_file = create_zipfile(dir_path)
        response = client.create_function(
            FunctionName=function_name,
            Runtime=runtime,
            Role=role,
            Handler='lambda_function.lambda_handler',
            Code={
                'ZipFile': open(zip_file, 'rb').read()
            },
            Description='autmatically uploaded',
            Timeout=5,
            MemorySize=128,
            Publish=True
        )
        http_resp = response["ResponseMetadata"]["HTTPStatusCode"]
        if http_resp == 201 and"FunctionArn" in response:
            fun_arn = response["FunctionArn"]
        logger.debug(response)
        response = client.add_permission(
            FunctionName=function_name,
            StatementId='1',
            Action='lambda:invokeFunction',
            Principal='alexa-appkit.amazon.com'
        )
        logger.debug(response)
    # print(response)
    except Exception as e:
        print(str(e))
    finally:
        # remove the temp dir containing the zipfile
        if os.path.exists(zip_file):
            shutil.rmtree(os.path.dirname(zip_file))
    return fun_arn


def update_lambda(client, function_name, dir_path):
    logger = logging.getLogger(__name__)
    zip_file = ""
    try:
        zip_file = create_zipfile(dir_path)
        zip_file_handle = open(zip_file, 'rb')
        response = client.update_function_code(
            FunctionName=function_name,
            ZipFile=zip_file_handle.read(),
            Publish=True
        )
        logger.debug(response)
    except Exception as e:
        print(str(e))
    finally:
        # remove the temp dir containing the zipfile
        if os.path.exists(zip_file):
            shutil.rmtree(os.path.dirname(zip_file))


def replace_nested_dict_val(orig_dict, k, v):
    for key, val in orig_dict.items():
        if key == k:
            orig_dict[key] = v
        elif isinstance(val, collections.Mapping):
            replace_nested_dict_val(orig_dict.get(key, {}), k, v)
    return orig_dict


def generate_testevents(test_data, event_template):
    event_list = []
    with open(test_data) as f:
        tests_list = json.load(f)
    with open(event_template) as f:
        template = json.load(f)
    for test in tests_list:
        new_event = copy.deepcopy(template)
        for k, v in test.items():
            new_event = replace_nested_dict_val(new_event, k, v)
        event_list.append(copy.deepcopy(new_event))
    return event_list


def run_all_tests(client, function_name, alexa_event_data, event_template):
    event_list = generate_testevents(alexa_event_data, event_template)
    for event in event_list:
        reply_text = test_lambda(client, function_name, event)
        if reply_text:
            print("Lambda function replied: " + str(reply_text))
        else:
            print("Lambda function didn't reply!!!")
        time.sleep(1)


def test_lambda(client, function_name, event):
        reply_text = None
        print("Sending Alexa Intent: " + event["request"]["intent"]["name"] +
              " and slots:" + str(event["request"]["intent"]["slots"]))
        event_binary = bytearray()
        event_binary.extend(json.dumps(event))
        response = client.invoke(
            FunctionName=function_name,
            InvocationType='RequestResponse',
            LogType='Tail',
            Payload=event_binary
        )
        try:
            res = response["Payload"].read()
            reply_json = json.loads(res)
            reply_text = reply_json["response"]["outputSpeech"]["text"]
        except Exception:
            print("Lambda function returned an error:")
            print_nested(reply_json)
        return reply_text


def print_nested(obj, nested_level=0, output=sys.stdout):
    spacing = '   '
    if type(obj) == dict:
        # print >> output, '%s{' % ((nested_level) * spacing)
        for k, v in obj.items():
            if hasattr(v, '__iter__'):
                print >> output, '%s%s:' % ((nested_level + 1) * spacing, k)
                print_nested(v, nested_level + 1, output)
            else:
                print >> output, '%s%s: %s' % ((nested_level + 1) * spacing, k, v)
        # print >> output, '%s}' % (nested_level * spacing)
    elif type(obj) == list:
        print >> output, '%s' % ((nested_level) * spacing)
        for v in obj:
            if hasattr(v, '__iter__'):
                print_nested(v, nested_level + 1, output)
            else:
                print >> output, '%s%s' % ((nested_level + 1) * spacing, v)
        # print >> output, '%s' % ((nested_level) * spacing)
    else:
        print >> output, '%s%s' % (nested_level * spacing, obj)



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
import os
from os import path
import sys
import shutil
import glob
from setuptools.command.test import test as TestCommand

base_dir = os.path.dirname(__file__)
src_dir = os.path.join(base_dir, "src")

# Get the long description from the README file
with open(path.join(base_dir, 'README.rst')) as f:
    long_description = f.read()


class ToxCommand(TestCommand):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]

    def initialize_options(self):
        """Initialize options and set their defaults."""
        TestCommand.initialize_options(self)
        self.tox_args = ''

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        import shlex
        errcode = tox.cmdline(args=shlex.split(self.tox_args))
        sys.exit(errcode)


class CleanCommand(TestCommand):
    description = "Custom clean command that forcefully removes dist/build directories"
    user_options = []

    def initialize_options(self):
        self.cwd = None

    def finalize_options(self):
        self.cwd = os.getcwd()

    def run(self):
        assert os.getcwd() == self.cwd, 'Must be in package root: %s' % self.cwd

        # List of things to remove
        rm_list = list()

        # Find any .pyc files or __pycache__ dirs
        for root, dirs, files in os.walk(self.cwd, topdown=False):
            for fname in files:
                if fname.endswith('.pyc') and os.path.isfile(os.path.join(root, fname)):
                    rm_list.append(os.path.join(root, fname))
            if root.endswith('__pycache__'):
                rm_list.append(root)

        # Find egg's
        for egg_dir in glob.glob('*.egg') + glob.glob('*egg-info'):
            rm_list.append(egg_dir)

        # Zap!
        for rm in rm_list:
            if self.verbose:
                print "Removing '%s'" % rm
            if os.path.isdir(rm):
                if not self.dry_run:
                    shutil.rmtree(rm)
            else:
                if not self.dry_run:
                    os.remove(rm)


setup(
    name='FirstAlexaSkills',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.1.1',

    description='A package for learning first steps with Alexa skills',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/means-to-meaning/FirstAlexaSkills',

    # Author details
    author='Vlastimil Pis',
    author_email='vlasto.pis@gmail.com',

    # Choose your license
    license='Amazon Software license',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Education',
        'Topic :: Home Automation',

        # Pick your license as you wish (should match "license" above)
        'License :: Other/Proprietary License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7'
    ],

    # What does your project relate to?
    keywords='alexa skills echo iot aws lambda',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    # include_package_data=True,

    # this does not work in development mode!!!
    # packages=['FirstAlexaSkills'],
    # package_dir={'FirstAlexaSkills': 'src/FirstAlexaSkills'},


    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['awscli', 'boto3', 'AWSIoTPythonSDK'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    tests_require=['tox', 'nose'],
    cmdclass={'test': ToxCommand,
              'clean': CleanCommand},

    extras_require={
        'dev': ['check-manifest', 'sphinx', 'autodoc'],
        'test': ['nose'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={'FirstAlexaSkills': ['data/event_template.json',
                                         'data/example_skills.zip']},

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
    scripts=['bin/unpack_example_skills',
             'bin/update_lambda_function',
             'bin/create_lambda_function',
             'bin/test_lambda_function']
)
